/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MsgsPerSecondDataProd.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.cewolf.dataproducers;

import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.TIME;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.util.StringHelper;

import de.laures.cewolf.DatasetProduceException;
import de.laures.cewolf.DatasetProducer;

public final class MsgsPerSecondDataProd implements DatasetProducer {

    /**
     * 
     */
    private static final long serialVersionUID = -1424547051372969763L;

    private static final String actionTitle = "show PEtALS messages per second repartition chart";

    final Logger log = Logger.getLogger(MsgsPerSecondDataProd.class);

    public MsgsPerSecondDataProd() {
        super();
    }

    /**
     * Returns the id of the producer
     * 
     * @return an id as a String
     */
    public String getProducerId() {
        return new String("MsgsPerSecondDataset Producer");
    }

    /**
     * Indicates if dataset produced has expired. method is called by the Cewolf
     * framework to check if a formerly produced data can be reused. As datas
     * are evolving and retrieved in an MBean, method always returns true to
     * force reloading datas.
     * 
     * @return true if the data which had been produced with the passed in
     *         parameters has expired since its creation, false otherwise
     */
    @SuppressWarnings("unchecked")
    public boolean hasExpired(Map params, Date since) {
        return true;
    }

    @SuppressWarnings("unchecked")
    public Object produceDataset(Map params) throws DatasetProduceException {
        // getting parameters
        final String seriesTitle = (String) params.get("seriesTitle");
        final String endpoint = (String) params.get("endpoint");
        final String component = (String) params.get("component");
        final String server = (String) params.get("server");
        final long sinceTime = ((Long) params.get("sinceTime")).longValue();

        final Map<Long, Long> msgPerSecond = new HashMap<Long, Long>();

        // creating period timestamps
        final long endTime = sinceTime - 5000 + (60000 * 5);

        try {
            final CollectorServer collector = CollectorManager.getCollector();

            List<Map<String, Object>> exchanges = null;

            if (StringHelper.isNotNullAndNotEmpty(endpoint)) {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshotsByServerAndComponentAndEndpoint(server,
                                component, endpoint);
            } else if (StringHelper.isNotNullAndNotEmpty(component)) {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshotsByServerAndComponent(server, component);
            } else if (StringHelper.isNotNullAndNotEmpty(server)) {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshotsByServer(server);
            } else {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshots();
            }
            constructTimeMap(msgPerSecond, exchanges);

            /**
             * as the exception is managed by the chart, we have to log and
             * throw it*
             */
        } catch (final DataCollectorServerException e) {
            this.log.error("Can't " + MsgsPerSecondDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (final DataCollectorClientException e) {
            this.log.error("Can't " + MsgsPerSecondDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.log.error("Can't " + MsgsPerSecondDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        }

        // dataset containing one or more serie(s) of time datas
        final TimeSeriesCollection dataset = new TimeSeriesCollection();

        // receive a series of time datas
        final TimeSeries timeDatas = new TimeSeries(seriesTitle, Second.class);
        for (long timestamp = (sinceTime / 1000) * 1000; timestamp <= endTime; timestamp += 1000) {
            long currentCount = 0L;
            if (msgPerSecond.containsKey(timestamp)) {
                currentCount = msgPerSecond.get(timestamp);
            }
            timeDatas.add(new Second(new Date(timestamp)), currentCount);
        }

        // adding series to dataset
        dataset.addSeries(timeDatas);

        // calendar used to format dates
        // Calendar cal = Calendar.getInstance();

        return dataset;
    }

    private void constructTimeMap(Map<Long, Long> msgPerSecond, List<Map<String, Object>> exchanges) {
        for (final Map<String, Object> mes : exchanges) {
            if (mes.get(TIME) != null) {
                final long time = ((Long) mes.get(TIME)).longValue();
                final long roundedTime = (time / 1000) * 1000;
                if (msgPerSecond.containsKey(roundedTime)) {
                    msgPerSecond.put(roundedTime, msgPerSecond.get(roundedTime) + 1L);
                } else {
                    msgPerSecond.put(roundedTime, 1L);
                }
            }
        }

    }
}
